package bushfire.estimator;

import bushfire.model.Model;


/**
 * This class represents a single weighted particle in the particle filter.
 * 
 * @author malcolmr
 */
public class Particle {

	protected Model myModel; 
	private double myWeight;

	public Particle(Model model) {
		myModel = Model.copyModel(model);
		myWeight = 1.0;
	}

	/**
	 * Copy constructor
	 * @param particle
	 */
	public Particle(Particle particle) {
		myModel = Model.copyModel(particle.myModel);
		myWeight = particle.myWeight;;
	}

	/**
	 * 
	 */
	public void nextStep() {
		myModel.nextStep();
	}

	/**
	 * @param x
	 * @param y
	 * @return
	 */
	public byte getFireAt(int x, int y) {
		return myModel.getFireAt(x, y);
	}

	/**
	 * @param x
	 * @param y
	 * @param state
	 */
	public void setFireAt(int x, int y, byte state) {
		myModel.setFireAt(x, y, state);
	}

	/**
	 * @param m
	 * @return
	 */
	public int distance(Particle p) {
		return myModel.distance(p.myModel);
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		return myModel.equals(obj);
	}


	/**
	 * @param x
	 * @param y
	 * @param observation
	 * @return
	 */
	public double observationProbability(int x, int y, byte observation) {
		return myModel.observationProbability(x, y, observation);
	}

	/**
	 * @return
	 */
	public double getWeight() {
		return myWeight;
	}

	/**
	 * @param d
	 */
	public void setWeight(double w) {
		myWeight = w;
	}

}
