package bushfire.estimator;

import java.awt.Dimension;
import java.awt.Point;

import bushfire.model.Model;

/**
 * A single particle in a TiledEstimator
 * 
 * @see TiledEstimator
 * @author malcolmr
 */
public class TiledParticle extends Particle {

	private Point myOffset;
	private Dimension myDimension;
	private double[][] myWeights;

	/**
	 * @param model
	 */
	public TiledParticle(Model model, Dimension dim, Point offset) {
		super(model);
		myDimension = dim;
		myOffset = offset;
		myWeights = new double[dim.width][dim.height];
		
		for (int x = 0; x < myWeights.length; x++) {
			for (int y = 0; y < myWeights[x].length; y++) {
				myWeights[x][y]	= 1.0;	
			}
		}
	}
	
	public double getTileWeight(Point tile) {
		return myWeights[tile.x][tile.y]; 
	}
	
	/**
	 * @param d
	 */
	public void setTileWeight(Point tile, double w) {
		myWeights[tile.x][tile.y] = w;
	}

	/**
	 * Copy the desingated tile from the source into this particle.
	 * 
	 * @param duplicatedParticle
	 * @param tile
	 */
	public void copyTile(TiledParticle source, Point tile) {
		int tileWidth = 1 + (myModel.getWidth() - 1) / myDimension.width;
		int tileHeight = 1 + (myModel.getHeight() - 1) / myDimension.height;
		
		int ox = tile.x * tileWidth + myOffset.x;
		int oy = tile.y * tileHeight + myOffset.y;
		
		for (int dx = 0; dx < tileWidth; dx++) {
			for (int dy = 0; dy < tileHeight; dy++) {
				if (myModel.onMap(ox + dx, oy + dy)) {
					byte state = source.getFireAt(ox + dx, oy + dy);
					setFireAt(ox + dx, oy + dy, state);				
				}
			}			
		}
	}
	
	/**
	 * @see bushfire.estimator.Particle#getWeight()
	 */
	public double getWeight() {
		throw new RuntimeException("TiledParticle.getWeight is not implemented.");
	}

	/**
	 * @see bushfire.estimator.Particle#setWeight(double)
	 */
	public void setWeight(double w) {
		throw new RuntimeException("TiledParticle.setWeight is not implemented.");
	}

}
