package bushfire.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ScrollPaneConstants;

import bushfire.estimator.CompositeEstimator;
import bushfire.estimator.Estimator;
import bushfire.estimator.TiledEstimator;
import bushfire.model.Model;

/**
 * The main class defined the user interface for the application.
 * It's an ugly hacked together thing, I know.
 * @author malcolmr
 */
public class GUI extends JFrame {

	private static final int SLEEP_STEP = 1000;

	private WholeNumberField myCatchProbField;
	private WholeNumberField myDieProbField;

	private JTextField[] myCellSupportField;
	private JTextField[] myCellProbField;

	private JComboBox myDrawMode;

	private JButton myResetButton;
	private JButton myStartButton;
	private JButton myNextButton;

	private final static String TITLE = "Bushfire";

	private Model myModel;
	private ModelView myModelView;
	private Estimator myEstimator;
	private EstimatorView myEstimatorView;

	private boolean myRunning = false;

	public GUI(Model model, Estimator estimator) {
		super(TITLE);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		myModel = model;
		myModelView = new ModelView(model);
		myEstimator = estimator;
		myEstimatorView = new EstimatorView(estimator);

		Container content = getContentPane();
		content.setLayout(new BorderLayout());

		JTabbedPane tabbedPane = new JTabbedPane();
		content.add(tabbedPane, BorderLayout.CENTER);

		JScrollPane scrollPane = new JScrollPane(myModelView);
		scrollPane.setAutoscrolls(true);
		scrollPane.setHorizontalScrollBarPolicy(
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(
			ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);

		Dimension viewSize = myModelView.getPreferredSize();

		scrollPane.setPreferredSize(
			new Dimension(viewSize.width + 5, viewSize.height + 5));
		tabbedPane.add("Model", scrollPane);

		scrollPane = new JScrollPane(myEstimatorView);
		scrollPane.setAutoscrolls(true);
		scrollPane.setHorizontalScrollBarPolicy(
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(
			ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);

		viewSize = myEstimatorView.getPreferredSize();

		scrollPane.setPreferredSize(
			new Dimension(viewSize.width + 5, viewSize.height + 5));
		tabbedPane.add("Estimator", scrollPane);

		// CONTROL PANEL

		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		BoxLayout layout = new BoxLayout(controlPanel, BoxLayout.Y_AXIS);
		controlPanel.setLayout(layout);

		myStartButton = new JButton("Run");
		controlPanel.add(myStartButton);

		myStartButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {

				synchronized (myStartButton) {
					setRunning(!myRunning);
				}
			}

		});

		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));

		myNextButton = new JButton("Next");
		controlPanel.add(myNextButton);

		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));

		myNextButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				Model.setCatchProb(myCatchProbField.getValue() / 100.0);
				Model.setDieProb(myDieProbField.getValue() / 100.0);
				myModel.nextStep();
			}

		});

		myResetButton = new JButton("Reset");
		controlPanel.add(myResetButton);

		myResetButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				myModel.reset();
				// myEstimator.reset() is called as a result of Estimator observing Model
				myEstimatorView.reset();
				setRunning(false);
			}

		});

		JPanel modelPanel = new JPanel();
		modelPanel.setBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.BLACK),
				"Model"));
		modelPanel.setBackground(Color.WHITE);
		BoxLayout mpLayout = new BoxLayout(modelPanel, BoxLayout.Y_AXIS);
		modelPanel.setLayout(mpLayout);
		controlPanel.add(modelPanel);

		JPanel estimatorPanel = new JPanel();
		estimatorPanel.setBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.BLACK),
				"Estimator"));
		estimatorPanel.setBackground(Color.WHITE);
		BoxLayout epLayout = new BoxLayout(estimatorPanel, BoxLayout.Y_AXIS);
		estimatorPanel.setLayout(epLayout);
		controlPanel.add(estimatorPanel);

		// MODEL PANEL

		modelPanel.add(Box.createRigidArea(new Dimension(0, 5)));
		JLabel label = new JLabel("Draw Mode:");
		modelPanel.add(label);

		ImageIcon[] images = new ImageIcon[3];
		images[0] = new ImageIcon();
		Image image = new BufferedImage(10, 10, BufferedImage.TYPE_INT_RGB);
		Graphics g = image.getGraphics();
		g.setColor(Color.GREEN);
		g.fillRect(0, 0, 10, 10);
		images[0].setImage(image);
		images[0].setDescription("Unburnt");

		images[1] = new ImageIcon();
		image = new BufferedImage(10, 10, BufferedImage.TYPE_INT_RGB);
		g = image.getGraphics();
		g.setColor(Color.RED);
		g.fillRect(0, 0, 10, 10);
		images[1].setImage(image);
		images[1].setDescription("Burning");

		images[2] = new ImageIcon();
		image = new BufferedImage(10, 10, BufferedImage.TYPE_INT_RGB);
		g = image.getGraphics();
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, 10, 10);
		images[2].setImage(image);
		images[2].setDescription("Burnt");

		myDrawMode = new JComboBox(images);
		myDrawMode.setBackground(Color.WHITE);
		ComboBoxRenderer renderer = new ComboBoxRenderer();
		renderer.setPreferredSize(new Dimension(80, 20));
		myDrawMode.setRenderer(renderer);
		myDrawMode.setMaximumRowCount(3);
		myDrawMode.setAlignmentX(Component.LEFT_ALIGNMENT);
		myDrawMode.setMaximumSize(new Dimension(600, 30));
		myDrawMode.setSelectedIndex(1);

		myDrawMode.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				switch (myDrawMode.getSelectedIndex()) {
					case 0 :
						myModelView.setCursorFireType(Model.UNBURNT);
						break;

					case 1 :
						myModelView.setCursorFireType(Model.BURNING);
						break;

					case 2 :
						myModelView.setCursorFireType(Model.BURNT);
						break;
				}
			}
		});

		modelPanel.add(myDrawMode);

		int catchPercent = (int) Math.floor(Model.getCatchProb() * 100);
		JPanel miniPanel = new JPanel();
		miniPanel.setLayout(new FlowLayout());
		miniPanel.setBackground(Color.WHITE);
		miniPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		miniPanel.add(new JLabel("Catch prob:"));
		myCatchProbField = new WholeNumberField(catchPercent, 2);
		miniPanel.add(myCatchProbField);
		miniPanel.add(new JLabel("%"));
		miniPanel.setMaximumSize(new Dimension(200, 20));
		modelPanel.add(miniPanel);

		myCatchProbField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				WholeNumberField input = (WholeNumberField) e.getSource();
				double prob = (double) input.getValue() / 100.0;
				Model.setCatchProb(prob);
			}
		});

		int burnPercent = (int) Math.floor(Model.getDieProb() * 100);
		miniPanel = new JPanel();
		miniPanel.setLayout(new FlowLayout());
		miniPanel.setBackground(Color.WHITE);
		miniPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		miniPanel.add(new JLabel("Die prob:"));
		myDieProbField = new WholeNumberField(burnPercent, 2);
		miniPanel.add(myDieProbField);
		miniPanel.add(new JLabel("%"));
		miniPanel.setMaximumSize(new Dimension(200, 20));
		modelPanel.add(miniPanel);

		myDieProbField.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				WholeNumberField input = (WholeNumberField) e.getSource();
				double prob = (double) input.getValue() / 100.0;
				Model.setDieProb(prob);
			}
		});

		modelPanel.add(Box.createVerticalGlue());

		// ESTIMATOR PANEL

		estimatorPanel.add(new JLabel("Cell state:"));
		
		myCellProbField = new JTextField[3];
		myCellSupportField = new JTextField[3];
		for (int i = 0; i < 3; i++) {
			myCellProbField[i] = new JTextField("0.0000");
			myCellSupportField[i] = new JTextField("0000");
			myCellProbField[i].setPreferredSize(new Dimension(50, 20));			
			myCellSupportField[i].setPreferredSize(new Dimension(50, 20));			
		}		

		miniPanel = new JPanel();
		miniPanel.setLayout(new FlowLayout());
		miniPanel.setBackground(Color.WHITE);
		miniPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		miniPanel.add(new JLabel("Unburnt:"));
		miniPanel.add(myCellProbField[Model.UNBURNT]);
		miniPanel.add(myCellSupportField[Model.UNBURNT]);
		miniPanel.setMaximumSize(new Dimension(200, 20));
		estimatorPanel.add(miniPanel);

		miniPanel = new JPanel();
		miniPanel.setLayout(new FlowLayout());
		miniPanel.setBackground(Color.WHITE);
		miniPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		miniPanel.add(new JLabel("Burning:"));
		miniPanel.add(myCellProbField[Model.BURNING]);
		miniPanel.add(myCellSupportField[Model.BURNING]);
		miniPanel.setMaximumSize(new Dimension(200, 20));
		estimatorPanel.add(miniPanel);

		miniPanel = new JPanel();
		miniPanel.setLayout(new FlowLayout());
		miniPanel.setBackground(Color.WHITE);
		miniPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		miniPanel.add(new JLabel("Burnt:"));
		miniPanel.add(myCellProbField[Model.BURNT]);
		miniPanel.add(myCellSupportField[Model.BURNT]);
		miniPanel.setMaximumSize(new Dimension(200, 20));
		estimatorPanel.add(miniPanel);

		myEstimatorView.addMouseMotionListener(new MouseMotionListener() {

			public void mouseDragged(MouseEvent e) {
				
			}

			public void mouseMoved(MouseEvent e) {
				Point p = e.getPoint();
				Point cell = myEstimatorView.pointToCell(p);
				if (cell != null) {
					for (int i = 0; i < 3; i++) {
						double prob = myEstimator.getProb(cell.x, cell.y, i);
						int percent = (int)Math.round(prob * 100);
						myCellProbField[i].setText(percent + "%");
						int support = myEstimator.getSupport(cell.x, cell.y, i);
						myCellSupportField[i].setText("" + support);
					}
				}
			}});
		

		JButton observeButton = new JButton("Observe");
		estimatorPanel.add(observeButton);
		
		observeButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				myEstimatorView.makeObservations();
			}});

		estimatorPanel.add(Box.createVerticalGlue());

		content.add(controlPanel, BorderLayout.EAST);

		pack();
	}

	public synchronized void setRunning(boolean running) {
		if (running) {
			myStartButton.setText("Stop");
		} else {
			myStartButton.setText("Run");
		}

		Model.setCatchProb(myCatchProbField.getValue() / 100.0);
		Model.setDieProb(myDieProbField.getValue() / 100.0);
		this.myRunning = running;
		notifyAll();
	}

	public void run() {

		int t = 0;
		while (true) {

			while (!myRunning) {
				try {
					synchronized (this) {
						wait();
					}
				} catch (InterruptedException e1) {
				}
			}

			System.out.println(t++);

			myModel.nextStep();

			try {
				Thread.sleep(SLEEP_STEP);
			} catch (InterruptedException e) {
				// ignore
			}
		}

	}

	public static void main(String[] args) {

		Model model = Model.makeModel(100, 75);
		
		CompositeEstimator estimator = new CompositeEstimator(model);
		estimator.addEstimator(new TiledEstimator(model, 250, 10, 10, 0, 0), 0.5);
		estimator.addEstimator(new TiledEstimator(model, 250, 10, 10, 0, 5), 0.5);
		estimator.addEstimator(new TiledEstimator(model, 250, 10, 10, 5, 0), 0.5);
		estimator.addEstimator(new TiledEstimator(model, 250, 10, 10, 5, 5), 0.5);

		GUI frame = new GUI(model, estimator);
		frame.pack();
		frame.setVisible(true);

		frame.run();

	}

	class ComboBoxRenderer extends JLabel implements ListCellRenderer {
		public ComboBoxRenderer() {
			setOpaque(true);
			setHorizontalAlignment(LEFT);
			setVerticalAlignment(CENTER);
		}

		public Component getListCellRendererComponent(
			JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus) {
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			ImageIcon icon = (ImageIcon) value;
			setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
			setText(icon.getDescription());
			setIcon(icon);
			return this;
		}
	}

}
